#
#    DeltaFi - Data transformation and enrichment platform
#
#    Copyright 2021-2025 DeltaFi Contributors <deltafi@deltafi.org>
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#

import io

import minio
import pytest
from deltafi.storage import Segment, ContentService
from mockito import when, mock, unstub, verifyStubbedInvocationsAreUsed, ANY

BUCKET = 'storage'
SEG_1_DATA = "one"
SEG_2_DATA = "twotwo"
TEST_DID = "123did"


def make_segment(segment_id, data):
    segment = Segment(uuid=segment_id, offset=0, size=len(data), did=TEST_DID)
    return segment


def make_segments():
    s1 = make_segment("seg1", SEG_1_DATA)
    s2 = make_segment("seg2", SEG_2_DATA)
    return [s1, s2]


def faux_content_service():
    return ContentService("http://127.0.0.1:6543", "access_key", "secret_key")


def test_content_service():
    unstub()
    minio_mock = mock(minio.Minio)
    when(minio_mock).bucket_exists(...).thenReturn(True)
    when(minio).Minio(...).thenReturn(minio_mock)
    service = faux_content_service()


def test_no_bucket():
    unstub()
    minio_mock = mock(minio.Minio)
    when(minio_mock).bucket_exists(...).thenReturn(False)
    when(minio).Minio(...).thenReturn(minio_mock)
    with pytest.raises(Exception):
        service = faux_content_service()


def test_put_str():
    unstub()

    minio_mock = mock(minio.Minio)
    when(minio_mock).bucket_exists(...).thenReturn(True)
    # return value is ignored in ContentService
    when(minio_mock).put_object(BUCKET, ANY(str), ANY(io.BytesIO), 8).thenReturn(None)
    when(minio).Minio(...).thenReturn(minio_mock)
    service = faux_content_service()

    segment = service.put_str("123did", "the-data")
    verifyStubbedInvocationsAreUsed(minio_mock)

    assert segment.offset == 0
    assert segment.size == 8
    assert segment.did == "123did"
    assert len(segment.uuid) == 36


def test_gett_str():
    unstub()

    bytes_one = SEG_1_DATA.encode('utf-8')
    bytes_two = SEG_2_DATA.encode('utf-8')

    minio_mock = mock(minio.Minio)
    when(minio_mock).bucket_exists(...).thenReturn(True)
    when(minio_mock).get_object(BUCKET, "123/123did/seg1", 0, len(SEG_1_DATA)).thenReturn(io.BytesIO(bytes_one))
    when(minio_mock).get_object(BUCKET, "123/123did/seg2", 0, len(SEG_2_DATA)).thenReturn(io.BytesIO(bytes_two))
    when(minio).Minio(...).thenReturn(minio_mock)
    service = faux_content_service()

    content = service.get_str(make_segments())
    verifyStubbedInvocationsAreUsed(minio_mock)

    assert content == SEG_1_DATA + SEG_2_DATA
    assert len(content) == 9


def test_delete_all():
    unstub()

    minio_mock = mock(minio.Minio)
    when(minio).Minio(...).thenReturn(minio_mock)
    when(minio_mock).bucket_exists(...).thenReturn(True)
    when(minio_mock).remove_objects(BUCKET, ANY()).thenReturn([])
    service = faux_content_service()
    service.delete_all(make_segments())

    verifyStubbedInvocationsAreUsed(minio_mock)



#
#    DeltaFi - Data transformation and enrichment platform
#
#    Copyright 2021-2025 DeltaFi Contributors <deltafi@deltafi.org>
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#


import pytest
import re

from deltafi.test_kit.compare_helpers import JsonCompareHelper


@pytest.mark.parametrize("obj, item",
                         [
                             ([{"a": "alpha"}, {"b": "bravo"}], "notfound"),                              # string
                             ([{"a": "alpha"}, {"b": "bravo"}], re.compile("^notfound$")),                # regex
                             ([{"a": "alpha"}, {"b": "bravo"}], ["notfound"]),                            # list of 1 string
                             ([{"a": "alpha"}, {"b": "bravo"}], ["notfound1", "notfound2"]),              # list of 2 strings
                             ([{"a": "alpha"}, {"b": "bravo"}], [re.compile("^notfound1$")]),             # list of 1 regex
                             ([{"a": "alpha"}, {"b": "bravo"}], [re.compile("^notfound1$"),
                                                                 re.compile("^notfound1$")]),             # list of 2 regexes
                             ([{"a": "alpha"}, {"b": "bravo"}], ["notfound1", re.compile("^notfound$")])  # list of string and regex
                         ])
def test_is_not_found_success(obj, item):
    json_compare_helper = JsonCompareHelper()
    assert json_compare_helper.is_not_found(obj, item) is None


@pytest.mark.parametrize("obj, item, in_err_msg, not_in_err_msg",
                         [
                             ([{"a": "alpha"}, {"b": "bravo"}], "bravo", ["bravo"], ["alpha"]),                  # string
                             ([{"a": "alpha"}, {"b": "bravo"}], re.compile("^bravo$"), ["bravo"], ["alpha"]),    # regex
                             ([{"a": "alpha"}, {"b": "bravo"}], ["bravo"], ["bravo"], ["alpha"]),                # list of 1 string
                             ([{"a": "alpha"}, {"b": "bravo"}], ["alpha", "bravo"], ["alpha", "bravo"], []),     # list of 2 strings
                             ([{"a": "alpha"}, {"b": "bravo"}], [re.compile("^bravo$")], ["bravo"], ["alpha"]),  # list of 1 regex
                             ([{"a": "alpha"}, {"b": "bravo"}], [re.compile("^bravo$"), re.compile("^alpha$")],
                              ["alpha", "bravo"], []),                                                           # list of 2 regexes
                             ([{"a": "alpha"}, {"b": "bravo"}], ["bravo", re.compile("^alpha$")],
                              ["alpha", "bravo"], []),                                                           # list of string and regex
                             ([{"target-key": "alpha"}, {"b": "bravo"}], "target-key", ["target-key"], []),      # demonstrate dict key
                         ])
def test_is_not_found_fail(obj, item, in_err_msg, not_in_err_msg):

    with pytest.raises(ValueError) as exc_info:
        json_compare_helper = JsonCompareHelper()
        json_compare_helper.is_not_found(obj, item)

    for err in in_err_msg:
        assert err in str(exc_info.value)

    for err in not_in_err_msg:
        assert err not in str(exc_info)


@pytest.mark.parametrize("obj, item",
                         [
                             ([{"a": "alpha"}, {"b": "bravo"}], "bravo"),                        # string
                             ([{"a": "alpha"}, {"b": "bravo"}], re.compile("^alp[a-z]{2}$")),    # regex
                             ([{"a": "alpha"}, {"b": "bravo"}], ["bravo"]),                      # list of 1 string
                             ([{"a": "alpha"}, {"b": "bravo"}], ["alpha", "bravo"]),             # list of 2 strings
                             ([{"a": "alpha"}, {"b": "bravo"}], [re.compile("^alp[a-z]{2}$")]),  # list of 1 regex
                             ([{"a": "alpha"}, {"b": "bravo"}], [re.compile("^alp[a-z]{2}$"),
                                                                 re.compile("^br[a-z]{3}$")]),   # list of 2 regexes
                             ([{"a": "alpha"}, {"b": "bravo"}],
                              ["alpha", "bravo", re.compile("^alpha$")]),                        # list of string and regex
                             ([{"target-key": "alpha"}, {"b": "bravo"}], "target-key")           # demonstrate dict key
                         ])
def test_is_found_success(obj, item):
    json_compare_helper = JsonCompareHelper()
    assert json_compare_helper.is_found(obj, item) is None


@pytest.mark.parametrize("obj, item, in_err_msg, not_in_err_msg",
                         [
                             ([{"a": "alpha"}, {"b": "bravo"}], "notfound", ["notfound"], ["alpha", "bravo"]),    # string
                             ([{"a": "alpha"}, {"b": "bravo"}], re.compile("^alp[a-z]{3}$"),
                              ["[re.compile('^alp[a-z]{3}$')]"], ["alpha", "bravo"]),                             # regex
                             ([{"a": "alpha"}, {"b": "bravo"}], ["notfound"], ["notfound"], ["alpha", "bravo"]),  # list of 1 string
                             ([{"a": "alpha"}, {"b": "bravo"}], ["notfound1", "notfound2"],
                              ["notfound1", "notfound2"], ["alpha", "bravo"]),                                    # list of 2 strings
                             ([{"a": "alpha"}, {"b": "bravo"}], [re.compile("^alp[a-z]{3}$")],
                              ["[re.compile('^alp[a-z]{3}$')]"], ["alpha", "bravo"]),                             # list of 1 regex
                             ([{"a": "alpha"}, {"b": "bravo"}],
                              [re.compile("^br[a-z]{3}$"), re.compile("^alp[a-z]{3}$")],
                              ["[re.compile('^alp[a-z]{3}$')]"], ["alpha", "bravo"]),                             # list of 2 regexes
                             ([{"a": "alpha"}, {"b": "bravo"}], ["bravo", re.compile("^alp[a-z]{3}$")],
                              ["[re.compile('^alp[a-z]{3}$')]"], ["alpha", "bravo"])                              # list of string and regex
                         ])
def test_is_found_fail(obj, item, in_err_msg, not_in_err_msg):

    with pytest.raises(ValueError) as exc_info:
        json_compare_helper = JsonCompareHelper()
        json_compare_helper.is_found(obj, item)

    for err in in_err_msg:
        assert err in str(exc_info.value)

    for err in not_in_err_msg:
        assert err not in str(exc_info)


@pytest.mark.parametrize("expected, actual",
                         [
                             ("{\"a\": 1, \"b\": 2}", "{\"a\": 1, \"b\": 2}"),  # string, same order
                             ("{\"a\": 1, \"b\": 2}", "{\"b\": 2, \"a\": 1}"),  # string, different order
                             ({"a": 1, "b": 2}, {"a": 1, "b": 2}),              # dict, same order
                             ({"a": 1, "b": 2}, {"b": 2, "a": 1})               # dict, different order
                         ])
def test_compare_success(expected, actual):
    json_compare_helper = JsonCompareHelper()
    assert json_compare_helper.compare(expected, actual, "test") is None


@pytest.mark.parametrize("expected, actual",
                         [
                             ("{\"a\": 1, \"b\": 2}", "{\"a\": 1, \"c\": 2}"),            # string, different property
                             ("{\"a\": 1, \"b\": 2}", "{\"a\": 2, \"b\": 2, \"c\": 3}"),  # string, additional property
                             ({"a": 1, "b": 2}, {"a": 1, "c": 2}),                        # dict, different property
                             ({"a": 1, "b": 2}, {"a": 1, "b": 2, "c": 3})                 # dict, additional property
                         ])
def test_compare_fail(expected, actual):
    with pytest.raises(ValueError):
        json_compare_helper = JsonCompareHelper()
        json_compare_helper.compare(expected, actual, "test")


@pytest.mark.parametrize("expected, actual",
                         [
                             # 'expected' as a list
                             ([1, 2, 3], [1, 2, 3]),                        # ints, same order
                             ([1, 2, 3], [1, 3, 2]),                        # ints, different order
                             ([1, 2, 3], [3, 2, 1]),                        # ints, different (complete reverse) order
                             ([1, 2, 2, 3], [1, 2, 2, 3]),                  # ints, repetitions in same order
                             ([1, 2, 2, 3], [2, 1, 2, 3]),                  # ints, repetitions in different order
                             (["a", "b", "c"], ["a", "b", "c"]),            # strings, same order
                             (["a", "b", "c"], ["a", "c", "b"]),            # strings, different order
                             (["a", "b", "b", "c"], ["a", "b", "b", "c"]),  # strings, repetitions in same order
                             (["a", "b", "b", "c"], ["b", "a", "b", "c"]),  # strings, repetitions in different order

                             # 'expected' as a map of repetitions
                             ({1: 1, 2: 1, 3: 1}, [1, 2, 3]),                  # ints, no repetitions
                             ({1: 1, 2: 2, 3: 1}, [1, 2, 2, 3]),               # ints, repetitions
                             ({"a": 1, "b": 1, "c": 1}, ["a", "b", "c"]),      # strings, no repetitions
                             ({"a": 1, "b": 2, "c": 1}, ["a", "b", "b", "c"])  # strings, repetitions
                         ])
def test_compare_lists_success(expected, actual):
    json_compare_helper = JsonCompareHelper()
    assert json_compare_helper.compare_lists(expected, actual, "test") is None


@pytest.mark.parametrize("expected, actual",
                         [
                             # 'expected' as a list
                             ([1, 2, 3], [1, 2, 3, 4]),                           # ints, extra item
                             ([1, 2, 3], [1, 3]),                                 # ints, missing item
                             ([1, 2, 3], [1, 3, 4]),                              # ints, missing item but same number of total items
                             ([1, 2, 3], [1, 2, 2, 3]),                           # ints, repeated item
                             ([1, 2, 2, 3], [1, 2, 2, 2, 3]),                     # ints, repeated item
                             (["a", "b", "c"], ["a", "b", "c", "d"]),             # strings, extra item
                             (["a", "b", "c"], ["a", "c"]),                       # strings, missing item
                             (["a", "b", "c"], ["a", "c", "d"]),                  # strings, missing item but same number of total items
                             (["a", "b", "c"], ["a", "b", "b", "c"]),             # strings, repeated item
                             (["a", "b", "b", "c"], ["a", "b", "b", "c", "b"]),   # strings, repeated item

                             # 'expected' as a map of repetitions
                             ({1: 1, 2: 1, 3: 1}, [1, 2, 3, 4]),        # ints, extra item
                             ({1: 1, 2: 1, 3: 1}, [1, 3]),              # ints, missing item
                             ({1: 1, 2: 1, 3: 1}, [1, 3, 4]),           # ints, missing item but same number of total items
                             ({1: 1, 2: 1, 3: 1}, [1, 2, 2, 3]),        # ints, repeated item
                             ({1: 1, 2: 2, 3: 1}, [1, 2, 2, 3, 2])      # ints, repeated item
                         ])
def test_compare_lists_fail(expected, actual):
    with pytest.raises(ValueError):
        json_compare_helper = JsonCompareHelper()
        json_compare_helper.compare_lists(expected, actual, "test")


@pytest.mark.parametrize("expected_subset, actual",
                         [
                             # 'expected_subset' as a list
                             ([1, 2, 3], [1, 3, 2]),                    # ints, 'expected' and 'actual' are the same, no repetitions
                             ([1, 2], [1, 3, 2]),                       # ints, 'expected' is subset of 'actual', no repetitions
                             ([1, 2, 2], [1, 2, 2]),                    # ints, 'expected' and 'actual' are the same, w/ repetitions
                             ([1, 2, 2], [1, 2, 3, 2]),                 # ints, 'expected' is subset of 'actual', w/ repetitions
                             (["a", "b", "c"], ["a", "c", "b"]),        # strings, 'expected' and 'actual' are the same, no repetitions
                             (["a", "b"], ["a", "c", "b"]),             # strings, 'expected' is a subset of 'actual', no repetitions
                             (["a", "b", "b"], ["a", "b", "b"]),        # strings, 'expected' and 'actual' are the same, w/ repetitions
                             (["a", "b", "b"], ["b", "a", "b", "c"]),   # strings, 'expected' is subset of 'actual', w/ repetitions

                             # 'expected_subset' as a map of repetitions
                             ({1: 1, 2: 1}, [1, 2, 3]),                 # ints, no repetitions
                             ({1: 1, 2: 2}, [1, 2, 3, 2]),              # ints, repetitions
                             ({"a": 1, "b": 1}, ["a", "b", "c"]),       # strings, no repetitions
                             ({"a": 1, "b": 2}, ["a", "b", "c", "b"])   # strings, repetitions
                         ])
def test_compare_lists_subset_success(expected_subset, actual):
    assert JsonCompareHelper.compare_lists_subset(expected_subset, actual) is None


@pytest.mark.parametrize("expected_subset, actual",
                         [
                             # 'expected_subset' as a list
                             ([1, 2, 3], [3, 2, 4]),                 # ints, missing item
                             ([1, 2], [1, 3, 2, 1]),                 # ints, additional repetition
                             (["a", "b", "c"], ["c", "b", "d"]),     # strings, missing item
                             (["a", "b"], ["a", "c", "b", "a"]),     # strings, additional item

                             # 'expected_subset' as a map of repetitions
                             ({1: 1, 2: 1, 3: 1}, [3, 2, 4]),               # ints, missing item
                             ({1: 1, 2: 2}, [1, 3, 2, 1]),                  # ints, additional item
                             ({"a": 1, "b": 1, "c": 1}, ["c", "b", "d"]),   # strings, missing item
                             ({"a": 1, "b": 1}, ["a", "c", "b", "a"])       # strings, additional item
                         ])
def test_compare_lists_subset_fail(expected_subset, actual):
    with pytest.raises(ValueError):
        JsonCompareHelper.compare_lists_subset(expected_subset, actual)


@pytest.mark.parametrize("expected_superset, actual",
                         [
                             # 'expected_superset' as a list
                             ([1, 2, 3], [1, 3, 2]),                    # ints, 'expected' and 'actual' are the same, no repetitions
                             ([1, 2, 3], [1, 3]),                       # ints, 'expected' is superset of 'actual', no repetitions
                             ([1, 2, 2], [2, 1, 2]),                    # ints, 'expected' and 'actual' are the same, w/ repetitions
                             ([1, 2, 2, 3], [1, 2, 3, 2]),              # ints, 'expected' is supersset of 'actual', w/ repetitions
                             (["a", "b", "c"], ["a", "c", "b"]),        # strings, 'expected' and 'actual' are the same, no repetitions
                             (["a", "b", "c"], ["a", "c"]),             # strings, 'expected' is a superset of 'actual', no repetitions
                             (["a", "b", "b"], ["b", "a", "b"]),        # strings, 'expected' and 'actual' are the same, w/ repetitions
                             (["a", "b", "b", "c"], ["b", "a", "b"]),   # strings, 'expected' is superset of 'actual', w/ repetitions

                             # 'expected_superset' as a map of repetitions
                             ({1: 1, 2: 1, 3: 1}, [1, 3, 2]),      # ints, 'expected' and 'actual' are the same, no repetitions
                             ({1: 1, 2: 1, 3: 1}, [1, 3]),         # ints, 'expected' is superset of 'actual', no repetitions
                             ({1: 1, 2: 2}, [1, 2, 2]),            # ints, 'expected' and 'actual' are the same, w/ repetitions
                             ({1: 1, 2: 2, 3: 1}, [1, 2, 3, 2]),   # ints, 'expected' is supersset of 'actual', w/ repetitions

                             ({"a": 1, "b": 1, "c": 1}, ["a", "c", "b"]),  # strings, 'expected' and 'actual' are the same, no repetitions
                             ({"a": 1, "b": 2, "c": 1}, ["a", "c"]),       # strings, 'expected' is a superset of 'actual', no repetitions
                             ({"a": 1, "b": 2}, ["b", "a", "b"]),          # strings, 'expected' and 'actual' are the same, w/ repetitions
                             ({"a": 1, "b": 2, "c": 1}, ["b", "a", "b"])   # strings, 'expected' is superset of 'actual', w/ repetitions
                         ])
def test_compare_lists_superset_success(expected_superset, actual):
    assert JsonCompareHelper.compare_lists_superset(expected_superset, actual) is None


@pytest.mark.parametrize("expected_superset, actual",
                         [
                             # 'expected_superset' as a list
                             ([1, 2, 3], [2, 1, 4]),               # ints, additional item
                             ([1, 2, 3], [1, 3, 2, 2]),            # ints, additional repetition
                             (["a", "b", "c"], ["b", "a", "d"]),   # strings, additional item
                             (["a", "b"], ["a", "b", "a"]),        # strings, additional repetition

                             # 'expected_superset' as a map of repetitions
                             ({1: 1, 2: 1, 3: 1}, [2, 1, 4]),               # ints, additional item
                             ({1: 1, 2: 1, 3: 1}, [1, 3, 2, 2]),            # ints, additional repetition
                             ({"a": 1, "b": 1, "c": 1}, ["b", "a", "d"]),   # strings, additional item
                             ({"a": 1, "b": 1}, ["a", "b", "a"])            # strings, additional repetition
                         ])
def test_compare_lists_superset_fail(expected_superset, actual):
    with pytest.raises(ValueError):
        JsonCompareHelper.compare_lists_superset(expected_superset, actual)


@pytest.mark.parametrize("regex, expected_result_list",
                         [
                             # regex as a string
                             ("^notfound--", []),                                                  # regex matches 0 of 4 items
                             ("^identity--", ["identity--678"]),                                   # regex matches 1 of 4 items
                             ("^malware--", ["malware--123", "malware--123456", "malware--354"]),  # regex matches 3 of 4 items
                             ("^malware--123", ["malware--123", "malware--123456"]),               # partial match from start of string
                             ("malware--123", ["malware--123", "malware--123456"]),                # partial match from anywhere in the string
                             ("^malware--123$", ["malware--123"]),                                 # exact match from start to end of string

                             # regex as a compiled pattern object
                             (re.compile("^notfound--"), []),                                                 # regex matches 0 of 4 items
                             (re.compile("^identity--"), ["identity--678"]),                                  # regex matches 1 of 4 items
                             (re.compile("^malware--"), ["malware--123", "malware--123456", "malware--354"])  # regex matches 3 of 4 items
                         ])
def test_create_list_from_list_using_filter_regex(regex, expected_result_list: list):
    source_list = ["malware--123", "malware--123456", "identity--678", "malware--354"]
    actual_result_list = JsonCompareHelper.create_list_from_list_using_filter_regex(regex, source_list)
    assert len(expected_result_list) == len(actual_result_list)

    # check that lists are equivalent including number of repetitions but ignoring order
    json_compare_helper = JsonCompareHelper()
    assert json_compare_helper.compare_lists(expected_result_list, actual_result_list, "test") is None



#
#    DeltaFi - Data transformation and enrichment platform
#
#    Copyright 2021-2025 DeltaFi Contributors <deltafi@deltafi.org>
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#

from importlib import metadata
from unittest import mock

from deltafi.plugin import Plugin, PluginCoordinates

from .sample.actions import SampleTransformAction


def test_plugin_registration_json(monkeypatch):
    set_mock_env(monkeypatch)
    plugin_coordinates = PluginCoordinates("plugin-group", "project-name", "1.0.0")
    plugin = Plugin("plugin-description", plugin_name="project-name",
                    plugin_coordinates=plugin_coordinates,
                    actions=[SampleTransformAction])

    assert get_expected_json() == plugin.registration_json()


def test_plugin_registration_json_from_env(monkeypatch):
    monkeypatch.setenv("PROJECT_GROUP", "plugin-group")
    monkeypatch.setenv("PROJECT_NAME", "project-name")
    monkeypatch.setenv("PROJECT_VERSION", "1.0.0")
    set_mock_env(monkeypatch)

    plugin = Plugin("plugin-description", actions=[SampleTransformAction])

    assert get_expected_json() == plugin.registration_json()


def test_plugin_find_actions(monkeypatch):
    set_mock_env(monkeypatch)
    plugin_coordinates = PluginCoordinates("plugin-group", "project-name", "1.0.0")
    plugin = Plugin("plugin-description", plugin_name="project-name",
                    plugin_coordinates=plugin_coordinates,
                    action_package="test.sample")

    assert get_expected_json() == plugin.registration_json()


def check_results(results: list):
    found = 0
    for r in results:
        if r['name'] == "name1":
            assert r['description'] == 'd1'
            found += 1000
        elif r['name'] == "name2":
            assert r['description'] == 'd2'
            found += 100
        elif r['name'] == "name3":
            assert r['description'] == 'd3'
            found += 10
        elif r['name'] == "name4":
            assert r['description'] == 'd4'
            found += 1
    return found


def test_plugin_load_variables_single_doc(monkeypatch):
    set_mock_env(monkeypatch)
    variables = Plugin.load_variables("test/plugin_data/flows", ["variables.yaml"])
    assert len(variables) == 3
    assert check_results(variables) == 1110


def test_plugin_load_variables_multi_doc(monkeypatch):
    set_mock_env(monkeypatch)
    variables = Plugin.load_variables("test/plugin_data/flows", ["variables.yml"])
    assert len(variables) == 3
    assert check_results(variables) == 1110


def test_plugin_integration_tests(monkeypatch):
    set_mock_env(monkeypatch)
    results = Plugin.load_integration_tests("test/plugin_data/tests")
    assert len(results) == 4
    assert check_results(results) == 1111


def test_plugin_register(monkeypatch):
    set_mock_env(monkeypatch)
    plugin_coordinates = PluginCoordinates("plugin-group", "project-name", "1.0.0")
    plugin = Plugin("plugin-description", plugin_name="project-name",
                    plugin_coordinates=plugin_coordinates,
                    action_package="test.sample")
    with mock.patch("requests.post") as mock_post:
        plugin._register()

    mock_post.assert_called_once_with("http://core/plugins",
                                      headers={'Content-type': 'application/json'},
                                      json=get_expected_json())


def set_mock_env(monkeypatch):
    monkeypatch.setenv("CORE_URL", "http://core")
    monkeypatch.setenv("IMAGE", "docker.io/group/plugin:1.0.0")
    monkeypatch.setenv("IMAGE_PULL_SECRET", "docker-creds")


def get_expected_json():
    expected_kit_version = metadata.version('deltafi')
    return {
        "pluginCoordinates": {
            "groupId": "plugin-group",
            "artifactId": "project-name",
            "version": "1.0.0"
        },
        "displayName": "project-name",
        "description": "plugin-description",
        "actionKitVersion": expected_kit_version,
        "image": "docker.io/group/plugin:1.0.0",
        "imagePullSecret": "docker-creds",
        "dependencies": [

        ],
        "actions": [
            {
                "name": "plugin-group.SampleTransformAction",
                "type": "TRANSFORM",
                "supportsJoin": False,
                "schema": {
                    "title": "SampleTransformParameters",
                    "type": "object",
                    "properties": {
                        "a_string": {
                            "title": "A String",
                            "description": "this string parameter is required",
                            "type": "string"
                        },
                        "def_string": {
                            "title": "Def String",
                            "default": "default-val",
                            "description": "str with default",
                            "type": "string"
                        },
                        "a_dict": {
                            "additionalProperties": {
                                "type": "string"
                            },
                            "title": "A Dict",
                            "description": "this dict parameter is required",
                            "type": "object"
                        },
                        "def_dict": {
                            "additionalProperties": {
                                "type": "string"
                            },
                            "title": "Def Dict",
                            "description": "dict has default",
                            "default": ["key1:val1"],
                            "type": "object"
                        },
                        "a_list": {
                            "items": {
                                "type": "string"
                            },
                            "title": "A List",
                            "description": "list with default",
                            "default": [],
                            "type": "array"
                        },
                        "a_bool": {
                            "title": "A Bool",
                            "description": "this boolean parameter is required",
                            "type": "boolean"
                        },
                        "def_int": {
                            "title": "Def Int",
                            "description": "int with default",
                            "default": 100,
                            "type": "integer"
                        }
                    },
                    "required": [
                        "a_string",
                        "a_dict",
                        "a_bool"
                    ]
                },
                "actionOptions": {
                    "description": "Transform action description",
                    "details": "The details",
                    "errors": [{
                        "conditions": ["Condition A", "Condition B"],
                        "description": "Error 1",
                    }, {
                        "description": "Error 2",
                    }],
                    "filters": [{
                        "conditions": ["Condition A", "Condition B"],
                        "description": "Filter 1",
                    }, {
                        "description": "Filter 2",
                    }],
                    "inputSpec": {
                        "contentSummary": "The input content summary",
                        "metadataSummary": "The input metadata summary",
                    },
                    "notes": ["Note 1", "Note2"],
                    "outputSpec": {
                        "annotationsSummary": "The output annotations summary",
                        "contentSummary": "The output content summary",
                        "metadataSummary": "The output metadata summary",
                        "passthrough": False,
                    },
                },
            }
        ],
        "variables": [

        ],
        "flowPlans": [

        ],
        "integrationTests": [

        ]
    }



#
#    DeltaFi - Data transformation and enrichment platform
#
#    Copyright 2021-2025 DeltaFi Contributors <deltafi@deltafi.org>
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#

from deltafi.result import EgressResult

from .assertions import *
from .framework import TestCaseBase, ActionTest


class EgressTestCase(TestCaseBase):
    def __init__(self, fields: Dict):
        super().__init__(fields)

    def expect_egress_result(self):
        self.expected_result_type = EgressResult


class EgressActionTest(ActionTest):
    def __init__(self, package_name: str):
        """
        Provides structure for testing DeltaFi Egress action
        Args:
            package_name: name of the actions package for finding resources
        """
        super().__init__(package_name)

    def egress(self, test_case: EgressTestCase):
        if test_case.expected_result_type == EgressResult:
            self.expect_egress_result(test_case)
        else:
            super().execute(test_case)

    def expect_egress_result(self, test_case: EgressTestCase):
        result = super().run_and_check_result_type(test_case, EgressResult)
        self.assert_egress_result(test_case, result)

    def assert_egress_result(self, test_case: EgressTestCase, result: EgressResult):
        self.compare_metrics(test_case.expected_metrics, result.metrics)
        self.compare_log_messages(test_case.expected_messages, result.messages)



#
#    DeltaFi - Data transformation and enrichment platform
#
#    Copyright 2021-2025 DeltaFi Contributors <deltafi@deltafi.org>
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#

from abc import abstractmethod
from typing import Dict, List

from deltafi.action import ActionOptions, DescriptionWithConditions, InputSpec, OutputSpec, TransformAction
from deltafi.domain import Context, Content
from deltafi.input import TransformInput
from deltafi.result import TransformResult
from deltafi.storage import ContentService
from mockito import mock
from pydantic import BaseModel, Field


class SampleTransformParameters(BaseModel):
    a_string: str = Field(description="this string parameter is required")
    def_string: str = Field(default="default-val", description="str with default")
    a_dict: dict[str, str] = Field(description="this dict parameter is required")
    def_dict: Dict[str, str] = Field(default={"key1:" "val1"}, description="dict has default")
    a_list: List[str] = Field(default=[], description="list with default")
    a_bool: bool = Field(description="this boolean parameter is required")
    def_int: int = Field(default=100, description="int with default")


class SampleTransformAction(TransformAction):
    def __init__(self):
        super().__init__('', ActionOptions(description='Transform action description',
                                           input_spec=InputSpec(content_summary='The input content summary', metadata_summary='The input metadata summary'),
                                           output_spec=OutputSpec(content_summary='The output content summary', metadata_summary='The output metadata summary', annotations_summary='The output annotations summary'),
                                           filters=[DescriptionWithConditions('Filter 1', ['Condition A', 'Condition B']),
                                                   'Filter 2'],
                                           errors=[DescriptionWithConditions('Error 1', ['Condition A', 'Condition B']),
                                                   'Error 2'],
                                           notes=['Note 1', 'Note2'],
                                           details='The details'))

    def param_class(self):
        return SampleTransformParameters

    def transform(self, context: Context, params: SampleTransformParameters, transform_input: TransformInput):
        return TransformResult(context).add_metadata('transformKey', 'transformValue') \
            .add_content(Content(name='transformed content', segments=[], media_type='text/plain',
                                 content_service=mock(ContentService)))


class SampleAbstractTransformAction(TransformAction):
    def __init__(self):
        super().__init__('Transform action description - ignored due to the abstract method')

    def param_class(self):
        return SampleTransformParameters

    @abstractmethod
    def extra_abstract_method(self):
        pass

    def transform(self, context: Context, params: SampleTransformParameters, transform_input: TransformInput):
        return TransformResult(context).add_metadata('transformKey', 'transformValue') \
            .add_content(Content(name='transformed content', segments=[], media_type='text/plain',
                                 content_service=mock(ContentService)))



#
#    DeltaFi - Data transformation and enrichment platform
#
#    Copyright 2021-2025 DeltaFi Contributors <deltafi@deltafi.org>
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#

from deltafi.domain import Context, Event
from deltafi.storage import Segment

SEG_ID = "1"
TEST_DID = "123did"


def make_segment(seg_id):
    return Segment(uuid=seg_id, offset=0, size=100, did=TEST_DID)


def make_context_dict():
    return {
        'did': TEST_DID,
        'deltaFileName': 'FILENAME',
        'dataSource': 'DATA_SOURCE',
        'flowName': 'FLOW_NAME',
        'flowId': 'FLOW_ID',
        'actionName': 'ACTION_NAME',
        'actionVersion': '1.0',
        'hostname': 'HOSTNAME',
        'systemName': 'SYSTEM_NAME'
    }


def make_content_dict(name):
    return {
        'name': name,
        'segments': [make_segment(SEG_ID).json()],
        'mediaType': 'xml'
    }


def make_delta_file_message_dict():
    return {
        'metadata': {'plKey1': 'valueA', 'plKey2': 'valueB'},
        'contentList': [make_content_dict('CONTENT_NAME'), make_content_dict('CONTENT 2')]
    }


def make_context():
    return Context(did=TEST_DID,
                   delta_file_name='FILENAME',
                   data_source='DATA_SOURCE',
                   flow_name='FLOW_NAME',
                   flow_id='FLOW_ID',
                   action_name='ACTION_NAME',
                   action_version='1.0',
                   hostname='HOSTNAME',
                   system_name='SYSTEM_NAME',
                   content_service=None)


def make_event(content_service):
    logger = None
    event = Event.create({
        'deltaFileMessages': [make_delta_file_message_dict()],
        'actionContext': make_context_dict(),
        'actionParams': {
            "thing": "theThing"
        }
    }, content_service, logger)
    return event



#
#    DeltaFi - Data transformation and enrichment platform
#
#    Copyright 2021-2025 DeltaFi Contributors <deltafi@deltafi.org>
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#

from typing import List

from deltafi.result import TransformResult, TransformResults

from .assertions import *
from .framework import TestCaseBase, ActionTest, IOContent


class TransformTestCase(TestCaseBase):
    def __init__(self, fields: Dict):
        super().__init__(fields)
        self.results = []

    def expect_transform_result(self):
        self.expected_result_type = TransformResult

    def expect_transform_results(self):
        self.expected_result_type = TransformResults

    def add_transform_result(self, content: List[IOContent], metadata: Dict, delete_metadata_keys: List[str],
                             annotations: Dict, name: str = None):
        self.results.append(
            {
                'content': content,
                'metadata': metadata,
                'delete_metadata_keys': delete_metadata_keys,
                'annotations': annotations,
                'name': name
            }
        )


class TransformActionTest(ActionTest):
    def __init__(self, package_name: str):
        """
        Provides structure for testing DeltaFi Transform action
        Args:
            package_name: name of the actions package for finding resources
        """
        super().__init__(package_name)

    def transform(self, test_case: TransformTestCase):
        if test_case.expected_result_type == TransformResult:
            self.expect_transform_result(test_case)
        elif test_case.expected_result_type == TransformResults:
            self.expect_transform_results(test_case)
        else:
            super().execute(test_case)

    def expect_transform_result(self, test_case: TransformTestCase):
        result = super().run_and_check_result_type(test_case, TransformResult)
        self.assert_transform_result(test_case, result)

    def expect_transform_results(self, test_case: TransformTestCase):
        result = super().run_and_check_result_type(test_case, TransformResults)
        self.assert_transform_results(test_case, result)

    def assert_transform_results(self, test_case: TransformTestCase, result: TransformResults):
        assert_equal_len_with_label(test_case.results, result.child_results, "invalid child count")
        for index, child_result in enumerate(result.child_results):
            self.compare_one_transform_result(test_case, child_result, index)
            expected = test_case.results[index]
            if 'name' in expected:
                assert_equal_with_label(expected["name"], child_result.delta_file_name, f"name[{index}]")

    def assert_transform_result(self, test_case: TransformTestCase, result: TransformResult):
        self.compare_metrics(test_case.expected_metrics, result.metrics)
        self.compare_log_messages(test_case.expected_messages, result.messages)
        self.compare_one_transform_result(test_case, result, 0)

    def compare_one_transform_result(self, test_case: TransformTestCase, result: TransformResult, index: int):
        expected = test_case.results[index]

        # Check output
        self.compare_content_list(test_case.compare_tool, expected['content'], result.content)

        # Check metadata
        assert_keys_and_values(expected['metadata'], result.metadata)

        # Check deleted metadata
        for key in expected['delete_metadata_keys']:
            assert_key_in(key, result.delete_metadata_keys)

        # Check annotations
        assert_keys_and_values(expected['annotations'], result.annotations)



#
#    DeltaFi - Data transformation and enrichment platform
#
#    Copyright 2021-2025 DeltaFi Contributors <deltafi@deltafi.org>
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#

from abc import ABC, abstractmethod
from typing import Any, List

from pydantic import BaseModel

from deltafi.actiontype import ActionType
from deltafi.domain import DeltaFileMessage
from deltafi.genericmodel import GenericModel
from deltafi.input import EgressInput, TransformInput
from deltafi.result import *


class Join(ABC):
    def join(self, transform_inputs: List[TransformInput]):
        all_content = []
        all_metadata = {}
        for transform_input in transform_inputs:
            all_content += transform_input.content
            all_metadata.update(transform_input.metadata)
        return TransformInput(content=all_content, metadata=all_metadata)


class ContentSpec:
    name: str
    media_type: str
    description: str

    def __init__(self, name: str = None, media_type: str = None, description: str = None):
        self.name = name
        self.media_type = media_type
        self.description = description

    def json(self):
        json_dictionary = {}
        if self.name is not None:
            json_dictionary['name'] = self.name
        if self.media_type is not None:
            json_dictionary['mediaType'] = self.media_type
        if self.description is not None:
            json_dictionary['description'] = self.description
        return json_dictionary


class KeyedDescription:
    key: str
    description: str

    def __init__(self, key: str, description: str):
        self.key = key
        self.description = description

    def json(self):
        json_dictionary = {}
        if self.key is not None:
            json_dictionary['key'] = self.key
            json_dictionary['description'] = self.description
        return json_dictionary


class InputSpec:
    content_summary: str
    content_specs: List[ContentSpec]
    metadata_summary: str
    metadata_descriptions: List[KeyedDescription]

    def __init__(self, content_summary: str = None, content_specs: List[ContentSpec] = None,
                 metadata_summary: str = None, metadata_descriptions: List[KeyedDescription] = None):
        self.content_summary = content_summary
        self.content_specs = content_specs
        self.metadata_summary = metadata_summary
        self.metadata_descriptions = metadata_descriptions

    def json(self):
        json_dictionary = {}
        if self.content_summary is not None:
            json_dictionary['contentSummary'] = self.content_summary
        if self.content_specs is not None:
            json_dictionary['contentSpecs'] = [cs.json() for cs in self.content_specs]
        if self.metadata_summary is not None:
            json_dictionary['metadataSummary'] = self.metadata_summary
        if self.metadata_descriptions is not None:
            json_dictionary['metadataDescriptions'] = [md.json() for md in self.metadata_descriptions]
        return json_dictionary


class OutputSpec:
    content_summary: str
    content_specs: List[ContentSpec]
    metadata_summary: str
    metadata_descriptions: List[KeyedDescription]
    passthrough: bool
    annotations_summary: str
    annotation_descriptions: List[KeyedDescription]

    def __init__(self, content_summary: str = None, content_specs: List[ContentSpec] = None,
                 metadata_summary: str = None, metadata_descriptions: List[KeyedDescription] = None,
                 passthrough: bool = False, annotations_summary: str = None,
                 annotation_descriptions: List[KeyedDescription] = None):
        self.content_summary = content_summary
        self.content_specs = content_specs
        self.metadata_summary = metadata_summary
        self.metadata_descriptions = metadata_descriptions
        self.passthrough = passthrough
        self.annotations_summary = annotations_summary
        self.annotation_descriptions = annotation_descriptions

    def json(self):
        json_dictionary = {}
        if self.content_summary is not None:
            json_dictionary['contentSummary'] = self.content_summary
        if self.content_specs is not None:
            json_dictionary['contentSpecs'] = [cs.json() for cs in self.content_specs]
        if self.metadata_summary is not None:
            json_dictionary['metadataSummary'] = self.metadata_summary
        if self.metadata_descriptions is not None:
            json_dictionary['metadataDescriptions'] = [md.json() for md in self.metadata_descriptions]
        if self.passthrough is not None:
            json_dictionary['passthrough'] = self.passthrough
        if self.annotations_summary is not None:
            json_dictionary['annotationsSummary'] = self.annotations_summary
        if self.annotation_descriptions is not None:
            json_dictionary['annotationDescriptions'] = [ad.json() for ad in self.annotation_descriptions]
        return json_dictionary


class DescriptionWithConditions:
    description: str
    conditions: List[str]

    def __init__(self, description: str = None, conditions: List[str] = None):
        self.description = description
        self.conditions = conditions

    def json(self):
        json_dictionary = {}
        if self.description is not None:
            json_dictionary['description'] = self.description
        if self.conditions is not None:
            json_dictionary['conditions'] = [c for c in self.conditions]
        return json_dictionary


class ActionOptions:
    description: str
    input_spec: InputSpec
    output_spec: OutputSpec
    filters: List[DescriptionWithConditions] = None
    errors: List[DescriptionWithConditions] = None
    notes: List[str]
    details: str

    def __init__(self, description: str = None, input_spec: InputSpec = None, output_spec: OutputSpec = None,
                 filters: List = None, errors: List = None, notes: List[str] = None, details: str = None):
        self.description = description
        self.input_spec = input_spec
        self.output_spec = output_spec
        if filters is not None:
            self.filters = []
            for f in filters:
                if isinstance(f, DescriptionWithConditions):
                    self.filters.append(f)
                else:
                    self.filters.append(DescriptionWithConditions(description=f))
        if errors is not None:
            self.errors = []
            for e in errors:
                if isinstance(e, DescriptionWithConditions):
                    self.errors.append(e)
                else:
                    self.errors.append(DescriptionWithConditions(description=e))
        self.notes = notes
        self.details = details

    def json(self):
        json_dictionary = {}
        if self.description is not None:
            json_dictionary['description'] = self.description
        if self.input_spec is not None:
            json_dictionary['inputSpec'] = self.input_spec.json()
        if self.output_spec is not None:
            json_dictionary['outputSpec'] = self.output_spec.json()
        if self.filters is not None:
            json_dictionary['filters'] = [f.json() for f in self.filters]
        if self.errors is not None:
            json_dictionary['errors'] = [e.json() for e in self.errors]
        if self.notes is not None:
            json_dictionary['notes'] = [n for n in self.notes]
        if self.details is not None:
            json_dictionary['details'] = self.details
        return json_dictionary


class Action(ABC):
    def __init__(self, action_type: ActionType, description: str, valid_result_types: tuple,
                 action_options: ActionOptions = None):
        self.action_type = action_type
        if action_options is None:
            self.action_options = ActionOptions(description=description)
        else:
            self.action_options = action_options
        self.valid_result_types = valid_result_types

    @abstractmethod
    def build_input(self, context: Context, delta_file_message: DeltaFileMessage):
        pass

    def execute_join_action(self, event):
        raise RuntimeError(f"Join is not supported for {self.__class__.__name__}")

    @abstractmethod
    def execute(self, context: Context, action_input: Any, params: BaseModel):
        pass

    def execute_action(self, event):
        if event.delta_file_messages is None or not len(event.delta_file_messages):
            raise RuntimeError(f"Received event with no delta file messages for did {event.context.did}")
        if event.context.join is not None:
            result = self.execute_join_action(event)
        else:
            result = self.execute(
                event.context,
                self.build_input(event.context, event.delta_file_messages[0]),
                self.param_class().model_validate(event.params))

        self.validate_type(result)
        return result

    @staticmethod
    def param_class():
        """Factory method to create and return an empty GenericModel instance.

        All action parameter classes must inherit pydantic.BaseModel.
        Use of complex types in custom action parameter classes must specify
        the internal types when defined. E.g., dict[str, str], or List[str]

        Returns
        -------
        GenericModel
            an empty GenericModel instance
        """
        return GenericModel

    def validate_type(self, result):
        if not isinstance(result, self.valid_result_types):
            raise ValueError(f"{self.__class__.__name__} must return one of "
                             f"{[result_type.__name__ for result_type in self.valid_result_types]} "
                             f"but a {result.__class__.__name__} was returned")


class EgressAction(Action, ABC):
    def __init__(self, description: str, action_options: ActionOptions = None):
        super().__init__(ActionType.EGRESS, description, (EgressResult, ErrorResult, FilterResult), action_options)

    def build_input(self, context: Context, delta_file_message: DeltaFileMessage):
        content = None
        if delta_file_message.content_list is not None and len(delta_file_message.content_list) > 0:
            content = delta_file_message.content_list[0]
        return EgressInput(content=content, metadata=delta_file_message.metadata)

    @abstractmethod
    def egress(self, context: Context, params: BaseModel, egress_input: EgressInput):
        pass

    def execute(self, context: Context, egress_input: EgressInput, params: BaseModel):
        return self.egress(context, params, egress_input)


class TimedIngressAction(Action, ABC):
    def __init__(self, description: str, action_options: ActionOptions = None):
        super().__init__(ActionType.TIMED_INGRESS, description, (IngressResult, ErrorResult), action_options)

    def build_input(self, context: Context, delta_file_message: DeltaFileMessage):
        return None

    @abstractmethod
    def ingress(self, context: Context, params: BaseModel):
        pass

    def execute(self, context: Context, input_placeholder: Any, params: BaseModel):
        return self.ingress(context, params)


class TransformAction(Action, ABC):
    def __init__(self, description: str, action_options: ActionOptions = None):
        super().__init__(ActionType.TRANSFORM, description,
                         (TransformResult, TransformResults, ErrorResult, FilterResult), action_options)

    def build_input(self, context: Context, delta_file_message: DeltaFileMessage):
        return TransformInput(content=delta_file_message.content_list, metadata=delta_file_message.metadata)

    def execute_join_action(self, event):
        if isinstance(self, Join):
            return self.execute(
                event.context,
                self.join([self.build_input(event.context, delta_file_message)
                           for delta_file_message in event.delta_file_messages]),
                self.param_class().model_validate(event.params))
        else:
            super().execute_join_action(event)

    @abstractmethod
    def transform(self, context: Context, params: BaseModel, transform_input: TransformInput):
        pass

    def execute(self, context: Context, transform_input: TransformInput, params: BaseModel):
        return self.transform(context, params, transform_input)



#
#    DeltaFi - Data transformation and enrichment platform
#
#    Copyright 2021-2025 DeltaFi Contributors <deltafi@deltafi.org>
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#

from deltafi.domain import Content
from deltafi.result import EgressResult, ErrorResult, TransformResult, ChildTransformResult, TransformResults

from .helperutils import make_context, make_segment


def verify_metric(metric, name, value, tags):
    assert metric['name'] == name
    assert metric['value'] == value
    assert metric['tags'] == tags


def verify_no_metrics(result):
    metrics = [metric.json() for metric in result.metrics]
    assert len(metrics) == 0


def test_egress_result():
    result = EgressResult(make_context())
    assert result.result_key is None
    assert result.result_type == "EGRESS"
    assert result.response() is None
    metrics = [metric.json() for metric in result.metrics]
    verify_no_metrics(result)


def test_error_result():
    result = ErrorResult(make_context(), "errorCause", "errorContext")
    result.annotate('a', 'b')
    assert result.result_key == "error"
    assert result.result_type == "ERROR"
    verify_no_metrics(result)
    response = result.response()
    assert len(response.items()) == 3
    assert response.get('cause') == "errorCause"
    assert response.get('context') == "errorContext"
    assert response['annotations'] == {'a': 'b'}


def verify_all_metadata(item):
    metadata = item.get("metadata")
    assert len(metadata) == 3
    assert metadata["key1"] == "val1"
    assert metadata["key2"] == "val2"
    assert metadata["key3"] == "val3"


def add_canned_metadata(result):
    result.add_metadata("key1", "val1")
    result.add_metadata("key2", "val2")
    result.add_metadata("key3", "val3")


def make_content(content_service, name, seg_id):
    content = Content(name=name, segments=[make_segment(seg_id)], media_type="xml", content_service=content_service)
    return content


def test_transform_result():
    result = TransformResult(make_context())
    add_canned_metadata(result)
    result.add_content(make_content(None, "content1", "id1"))
    result.add_content(make_content(None, "content2", "id2"))
    result.annotate('a', 'b')
    result.delete_metadata_key('delete1')
    result.delete_metadata_key('delete2')

    assert result.result_key == "transform"
    assert result.result_type == "TRANSFORM"
    verify_no_metrics(result)

    response = result.response()
    assert len(response) == 1
    first = response[0]
    assert len(first) == 5
    verify_all_metadata(first)
    content = first.get("content")
    assert len(content) == 2
    assert content[0]['name'] == "content1"
    assert content[1]['name'] == "content2"
    assert first.get("annotations") == {'a': 'b'}
    assert first.get('deleteMetadataKeys') == ['delete1', 'delete2']

    assert len(result.get_segment_names()) == 2
    assert "123/123did/id1" in result.get_segment_names();
    assert "123/123did/id2" in result.get_segment_names();


def make_child(context, name=None):
    child1 = ChildTransformResult(context, name)
    add_canned_metadata(child1)
    child1.add_content(make_content(None, "content1", "id1"))
    child1.add_content(make_content(None, "content2", "id2"))
    child1.annotate('a', 'b')
    child1.delete_metadata_key('delete1')
    child1.delete_metadata_key('delete2')
    if name is not None:
        child1.log_info(f"My name is {name}")
    return child1


def test_transform_many_result():
    context = make_context()
    many_result = TransformResults(context)

    many_result.add_result(make_child(context, "name1"))
    many_result.add_result(make_child(context))
    many_result.add_result(make_child(context, "name3"))

    assert many_result.result_key == "transform"
    assert many_result.result_type == "TRANSFORM"
    verify_no_metrics(many_result)

    response = many_result.response()

    print(response)

    assert len(response) == 3

    first = response[0]
    assert len(first) == 7
    assert "name" in first
    assert first.get("name") == "name1"
    assert "messages" in first
    messages = first['messages']
    assert len(messages) == 1
    first_message = messages[0]
    assert "severity" in first_message;
    assert first_message["severity"] == "INFO"

    verify_all_metadata(first)
    content = first.get("content")
    assert len(content) == 2
    assert content[0]['name'] == "content1"
    assert content[1]['name'] == "content2"
    assert first.get("annotations") == {'a': 'b'}
    assert first.get('deleteMetadataKeys') == ['delete1', 'delete2']

    second = response[1]
    assert len(second) == 6
    assert "name" not in second
    assert "messages" in second
    messages = second['messages']
    assert len(messages) == 0

    third = response[2]
    assert len(third) == 7
    assert "name" in third
    assert third.get("name") == "name3"

    assert len(many_result.get_segment_names()) == 2
    assert "123/123did/id1" in many_result.get_segment_names();
    assert "123/123did/id2" in many_result.get_segment_names();
